﻿using GroupBar_Demo.Plugins.WpfTestPlugins;
using Syncfusion.Windows.Tools.Controls;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;

namespace GroupBar_Demo
{
    class MainViewModel :INotifyPropertyChanged
    {
        public MainViewModel()
        {
            Plugin1 plugin1 = new Plugin1();
            Plugin2 plugin2 = new Plugin2();

            PluginMenuList = new ObservableCollection<PluginMenuItem>
            {
                new PluginMenuItem { Title = "Plugin 1", PluginValue = plugin1},
                new PluginMenuItem { Title = "Plugin 2", PluginValue = plugin2},
                new PluginMenuItem { Title = "Plugin 3" }
            };

            SetUpSingleGroupMenu(PluginMenuList);
        }

        //Set up one group to display the plugins.
        private void SetUpSingleGroupMenu(ObservableCollection<PluginMenuItem> pluginMenuItems)
        {
            ObservableCollection<PluginGroup> pluginGroupList = new ObservableCollection<PluginGroup>();

            PluginGroup group = new PluginGroup("Plugin List");
            PluginGroup group2 = new PluginGroup("Group 2");

            foreach (PluginMenuItem pluginMenuItem in pluginMenuItems)
            {
                group.PluginMenuItems.Add(pluginMenuItem);
            }

            foreach (PluginMenuItem pluginMenuItem in pluginMenuItems)
            {
                group2.PluginMenuItems.Add(pluginMenuItem);
            }

            pluginGroupList.Add(group);
            pluginGroupList.Add(group2);
            PluginGroupList = pluginGroupList;
            PluginGroupList[0].IsSelected = true;
        }

        private ObservableCollection<PluginGroup> _pluginGroupList = new ObservableCollection<PluginGroup>();
        public ObservableCollection<PluginGroup> PluginGroupList
        {
            get { return _pluginGroupList; }
            set
            {
                if (_pluginGroupList != value)
                {
                    _pluginGroupList = value;
                    OnPropertyChanged(nameof(PluginGroupList));
                }
            }
        }

        private ObservableCollection<PluginMenuItem> _pluginMenuList = new ObservableCollection<PluginMenuItem>();
        public ObservableCollection<PluginMenuItem> PluginMenuList
        {
            get { return _pluginMenuList; }
            set
            {
                _pluginMenuList = value;
                OnPropertyChanged(nameof(PluginMenuList));
            }
        }

        private GroupViewItem _selectedGroupViewItem;

        public GroupViewItem SelectedGroupViewItem
        {
            get { return _selectedGroupViewItem; }
            set
            {
                //I would really like to know why this fires FOUR times when an item is selected (Should be twice, once to set the value and once to set back to null)
                _selectedGroupViewItem = value;
                if (_selectedGroupViewItem != null)
                {
                    SelectedMenuItem = (PluginMenuItem)_selectedGroupViewItem.DataContext;
                }
                OnPropertyChanged(nameof(SelectedGroupViewItem));
            }
        }

        private PluginMenuItem _selectedMenuItem;

        public PluginMenuItem SelectedMenuItem
        {
            get { return _selectedMenuItem; }
            set
            {
                if (_selectedMenuItem != value)
                {
                    _selectedMenuItem = value;
                    SelectedPluginChanged();
                    SelectedGroupViewItem = null;
                    OnPropertyChanged(nameof(SelectedMenuItem));
                }
            }
        }

        public void SelectedPluginChanged()
        {
            // Look for an existing tab with the required name
            var plugin = OpenPlugins.FirstOrDefault(x => x.Title == SelectedMenuItem.Title);

            // No plugin found - create a new instance
            if (plugin == null)
            {
                plugin = new Plugin(SelectedMenuItem.PluginValue);
                    
                OpenPlugins.Add(plugin);
            }

            SelectedPlugin = plugin;
        }

        private ObservableCollection<Plugin> _openPlugins = new ObservableCollection<Plugin>();
        public ObservableCollection<Plugin> OpenPlugins
        {
            get { return _openPlugins; }
            set
            {
                if (_openPlugins != value)
                {
                    _openPlugins = value;
                    OnPropertyChanged(nameof(OpenPlugins));
                }
            }
        }

        private Plugin _selectedPlugin;
        public Plugin SelectedPlugin
        {
            get { return _selectedPlugin; }
            set
            {
                if (_selectedPlugin != value)
                {
                    _selectedPlugin = value;
                    OnPropertyChanged(nameof(SelectedPlugin));
                }
            }
        }

        #region PropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string property)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(property));
        }

        #endregion
    }
}
